"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.JsonApiTransformerService = void 0;
const tslib_1 = require("tslib");
const common_1 = require("@nestjs/common");
const core_1 = require("@nestjs/core");
const json_api_nestjs_shared_1 = require("@klerick/json-api-nestjs-shared");
const route_path_factory_1 = require("@nestjs/core/router/route-path-factory");
const entity_param_map_service_1 = require("./entity-param-map.service");
function assertColumnName(entity, columnName) {
    const entityType = entity;
    if (!(columnName in entityType)) {
        throw new Error(`${columnName} not exist in ${entityType.name}`);
    }
}
(0, common_1.Injectable)();
class JsonApiTransformerService {
    applicationConfig;
    entityParamMapService;
    _urlPath;
    get currentMapProps() {
        return this.entityParamMapService.entityParaMap;
    }
    get urlPath() {
        if (this._urlPath)
            return [...this._urlPath];
        this._urlPath = [''];
        const prefix = this.applicationConfig.getGlobalPrefix();
        const version = this.applicationConfig.getVersioning();
        const routePathFactory = new route_path_factory_1.RoutePathFactory(this.applicationConfig);
        if (prefix) {
            this._urlPath.push(this.applicationConfig.getGlobalPrefix());
        }
        if (version && version.type === common_1.VersioningType.URI) {
            const firstVersion = Array.isArray(version.defaultVersion)
                ? version.defaultVersion[0]
                : version.defaultVersion;
            if (firstVersion) {
                this._urlPath.push(`${routePathFactory.getVersionPrefix(version)}${firstVersion.toString()}`);
            }
        }
        return [...this._urlPath];
    }
    transformData(data, query) {
        if (Array.isArray(data)) {
            const resultData = {
                data: data.map((item) => this.transformItem(item, this.currentMapProps, query)),
            };
            if (query.include) {
                resultData.included = this.extractIncluded(data, query);
            }
            return resultData;
        }
        const resultData = {
            data: this.transformItem(data, this.currentMapProps, query),
        };
        if (query.include) {
            resultData.included = this.extractIncluded([data], query);
        }
        return resultData;
    }
    transformItem(item, mapProps, query) {
        const { fields } = query;
        const targetField = fields && fields.target ? fields.target : null;
        assertColumnName(item, mapProps.primaryColumnName);
        const props = json_api_nestjs_shared_1.ObjectTyped.keys(mapProps.propsType).filter((i) => {
            if (i.toString() === mapProps.primaryColumnName.toString()) {
                return false;
            }
            if (!targetField) {
                return true;
            }
            return targetField.includes(i);
        });
        return {
            id: `${item[mapProps.primaryColumnName]}`,
            type: mapProps.typeName,
            attributes: this.extractAttributes(item, props),
            links: {
                self: this.getLink(mapProps.typeName, item[mapProps.primaryColumnName]),
            },
            relationships: this.transformRelationships(item, mapProps, query),
        };
    }
    transformRel(item, rel) {
        const relProps = Reflect.get(this.currentMapProps.relationProperty, rel);
        const relationMapPops = this.entityParamMapService.getParamMap(relProps.entityClass);
        if (!relationMapPops)
            throw new Error('Not found props map for ' + relProps.entityClass);
        const props = item[rel];
        if (Array.isArray(props)) {
            return props.map((i) => ({
                type: relationMapPops.typeName,
                id: i[relationMapPops.primaryColumnName].toString(),
            }));
        }
        else {
            assertColumnName(item[rel], relationMapPops.primaryColumnName);
            return props
                ? {
                    type: relationMapPops.typeName,
                    id: props[relationMapPops.primaryColumnName].toString(),
                }
                : null;
        }
    }
    transformRelationships(item, mapProps, query) {
        const { include } = query;
        const includeMap = new Map((include || []).map((i) => [i, true]));
        const primaryColumnName = mapProps.primaryColumnName;
        assertColumnName(item, primaryColumnName);
        return json_api_nestjs_shared_1.ObjectTyped.keys(mapProps.relationProperty).reduce((acum, i) => {
            acum[i] = {
                links: {
                    self: this.getLink(mapProps.typeName, item[primaryColumnName], 'relationships', i.toString()),
                },
            };
            if (includeMap.has(i)) {
                const relationMapPops = this.entityParamMapService.getParamMap(mapProps.relationProperty[i].entityClass);
                if (!relationMapPops)
                    throw new Error('Not found props map for ' +
                        mapProps.relationProperty[i].entityClass.name);
                if (mapProps.relationProperty[i].isArray) {
                    if (item[i] && Array.isArray(item[i]) && item[i].length > 0) {
                        // @ts-expect-error incorrect parse
                        acum[i]['data'] = item[i].map((rel) => ({
                            id: rel[relationMapPops.primaryColumnName].toString(),
                            type: relationMapPops.typeName,
                        }));
                    }
                    else {
                        // @ts-expect-error incorrect parse
                        acum[i]['data'] = [];
                    }
                }
                else {
                    const relPrimaryColumnName = relationMapPops.primaryColumnName;
                    const relType = item[i];
                    if (item[i]) {
                        assertColumnName(relType, relPrimaryColumnName);
                        // @ts-expect-error incorrect parse
                        acum[i]['data'] = {
                            id: `${relType[relPrimaryColumnName]}`,
                            type: relationMapPops.typeName,
                        };
                    }
                    else {
                        // @ts-expect-error incorrect parse
                        acum[i]['data'] = null;
                    }
                }
            }
            return acum;
        }, {});
    }
    extractAttributes(item, fields) {
        const mapFields = fields.reduce((acum, field) => {
            acum[field.toString()] = true;
            return acum;
        }, {});
        return json_api_nestjs_shared_1.ObjectTyped.entries(item).reduce((acum, [name, value]) => {
            if (name in mapFields && mapFields[name.toString()]) {
                // @ts-expect-error assign key to object entity
                acum[name] = value;
            }
            return acum;
        }, {});
    }
    extractIncluded(data, query) {
        const includeArray = [];
        const { include } = query;
        if (!include)
            return [];
        for (const relationPropsFromInclude of include) {
            const relationProps = this.currentMapProps.relationProperty[relationPropsFromInclude];
            if (!relationProps)
                continue;
            const relationMapProp = this.entityParamMapService.getParamMap(relationProps.entityClass);
            if (!relationMapProp)
                throw new Error('Not found props for relation ' +
                    relationPropsFromInclude +
                    'in' +
                    this.currentMapProps.className);
            const { fields } = query;
            const selectFieldForInclude = Reflect.get(fields || {}, relationPropsFromInclude);
            const queryForInclude = {
                ...query,
                fields: {
                    target: selectFieldForInclude &&
                        Array.isArray(selectFieldForInclude) &&
                        selectFieldForInclude.length > 0
                        ? selectFieldForInclude
                        : null,
                },
                include: null,
            };
            for (const dataItem of data) {
                const propRel = dataItem[relationPropsFromInclude];
                if (!propRel)
                    continue;
                if (Array.isArray(propRel)) {
                    for (const i of propRel) {
                        includeArray.push(this.transformItem(i, relationMapProp, queryForInclude));
                    }
                }
                else {
                    includeArray.push(this.transformItem(propRel, relationMapProp, queryForInclude));
                }
            }
        }
        return includeArray;
    }
    getLink(...partOfUrl) {
        const urlPath = this.urlPath;
        urlPath.push(...partOfUrl);
        return urlPath.join('/');
    }
}
exports.JsonApiTransformerService = JsonApiTransformerService;
tslib_1.__decorate([
    (0, common_1.Inject)(core_1.ApplicationConfig),
    tslib_1.__metadata("design:type", core_1.ApplicationConfig)
], JsonApiTransformerService.prototype, "applicationConfig", void 0);
tslib_1.__decorate([
    (0, common_1.Inject)(entity_param_map_service_1.EntityParamMapService),
    tslib_1.__metadata("design:type", entity_param_map_service_1.EntityParamMapService)
], JsonApiTransformerService.prototype, "entityParamMapService", void 0);
//# sourceMappingURL=json-api-transformer.service.js.map