"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getAll = getAll;
const swagger_1 = require("@nestjs/swagger");
const json_api_nestjs_shared_1 = require("@klerick/json-api-nestjs-shared");
const utils_1 = require("../utils");
const constants_1 = require("../../../../constants");
function getAll(controller, descriptor, entity, mapEntity, methodName) {
    const { props, relations, relationProperty, primaryColumnName } = mapEntity.getParamMap(entity);
    (0, utils_1.assertIsKeysOfObject)(entity, props);
    (0, utils_1.assertIsKeysOfObject)(entity, relations);
    const entityRelationStructure = {};
    const relationTree = json_api_nestjs_shared_1.ObjectTyped.entries(relationProperty).reduce((acum, [name, props]) => {
        const relMap = mapEntity.getParamMap(props.entityClass);
        acum.push(...relMap.props.map((i) => `${name.toLocaleString()}.${i}`));
        entityRelationStructure[name] = relMap.props;
        return acum;
    }, []);
    (0, swagger_1.ApiOperation)({
        summary: `Get list items of resource "${entity.name}"`,
        operationId: `${controller.constructor.name}_${methodName}`,
        servers: undefined,
    })(controller, methodName, descriptor);
    (0, swagger_1.ApiQuery)({
        name: 'fields',
        required: false,
        style: 'deepObject',
        schema: {
            type: 'object',
        },
        examples: {
            allField: {
                summary: 'Select all field',
                description: 'Select field for target and relation',
                value: {
                    target: props.join(','),
                    ...json_api_nestjs_shared_1.ObjectTyped.entries(entityRelationStructure).reduce((acum, [name, props]) => {
                        acum[name.toString()] = props.join(',');
                        return acum;
                    }, {}),
                },
            },
            selectOnlyIdsTarget: {
                summary: 'Select ids for target',
                description: 'Select ids for target',
                value: {
                    target: props.filter((i) => i === primaryColumnName).join(','),
                },
            },
        },
        description: `Object of field for select field from "${entity.name}" resource`,
    })(controller, methodName, descriptor);
    // https://github.com/OAI/OpenAPI-Specification/issues/1706#issuecomment-2704374644 need wait fix deepObject
    (0, swagger_1.ApiQuery)({
        name: 'filter',
        required: false,
        style: 'deepObject',
        schema: {
            type: 'object',
            properties: {
                ...props.reduce((acum, i) => {
                    Reflect.set(acum, String(i), {
                        type: 'object',
                        properties: Object.keys(json_api_nestjs_shared_1.FilterOperand).reduce((acum, name) => {
                            Reflect.set(acum, String(name), {
                                type: 'string',
                            });
                            return acum;
                        }, {}),
                        minProperties: 1,
                        additionalProperties: false,
                    });
                    return acum;
                }, {}),
                ...relationTree.reduce((acum, i) => {
                    Reflect.set(acum, String(i), {
                        type: 'object',
                        properties: Object.keys(json_api_nestjs_shared_1.FilterOperand).reduce((acum, name) => {
                            Reflect.set(acum, String(name), {
                                type: 'string',
                            });
                            return acum;
                        }, {}),
                        minProperties: 1,
                        additionalProperties: false,
                    });
                    return acum;
                }, {}),
            },
            additionalProperties: false,
        },
        examples: {
            simpleExample: {
                summary: 'Several conditional',
                description: 'Get if relation is not null',
                value: {
                    [props[0]]: {
                        in: '1,2,3',
                    },
                    [props[1]]: {
                        lt: '1',
                    },
                    [relationTree[0]]: {
                        eq: 'test',
                    },
                },
            },
            relationNull: {
                summary: 'Get if relation is null',
                description: 'Get if relation is null',
                value: {
                    [relations[0]]: {
                        eq: null,
                    },
                },
            },
            relationNotNull: {
                summary: 'Get if relation is not null',
                description: 'Get if relation is not null',
                value: {
                    [relations[0]]: {
                        ne: null,
                    },
                },
            },
            getRelationByConditional: {
                summary: 'Get if relation field is',
                description: 'Get if relation field is',
                value: {
                    [relationTree[0]]: {
                        eq: 'test',
                    },
                },
            },
        },
        description: `Object of filter for select items from "${entity.name}" resource`,
    })(controller, methodName, descriptor);
    let sortAscRelation = {};
    let sortDescRelation = {};
    let sortSeveral = {
        summary: 'Sort several field',
        description: 'Sort several field',
        value: `${String(props[1])},-${String(props[0])}`,
    };
    if (relations.length > 0) {
        (0, swagger_1.ApiQuery)({
            name: 'include',
            required: false,
            enum: relations,
            style: 'simple',
            isArray: true,
            description: `"${entity.name}" resource item has been extended with existing relations`,
            examples: {
                withInclude: {
                    summary: 'Add all relation',
                    description: 'Add all realtion',
                    value: relations,
                },
                without: {
                    summary: 'Without relation',
                    description: 'Without all relation',
                    value: [],
                },
            },
        })(controller, methodName, descriptor);
        sortAscRelation = {
            summary: 'Sort field relation by ASC',
            description: 'Sort field relation by ASC',
            value: relationTree[2],
        };
        sortDescRelation = {
            summary: 'Sort field relation by DESC',
            description: 'Sort field relation by DESC',
            value: `-${relationTree[2]}`,
        };
        sortSeveral = {
            summary: 'Sort several field with relation',
            description: 'Sort several field relation',
            value: `${String(props[1])},-${relationTree[2]},${relationTree[1]},-${String(props[0])}`,
        };
    }
    (0, swagger_1.ApiQuery)({
        name: 'sort',
        type: 'string',
        required: false,
        description: `Params for sorting of "${entity.name}"`,
        examples: {
            sortAsc: {
                summary: 'Sort field by ASC',
                description: 'Sort field by ASC',
                value: props[1],
            },
            sortDesc: {
                summary: 'Sort field by DESC',
                description: 'Sort field by DESC',
                value: `-${String(props[1])}`,
            },
            sortAscRelation,
            sortDescRelation,
            sortSeveral,
        },
    })(controller, methodName, descriptor);
    (0, swagger_1.ApiQuery)({
        name: 'page',
        style: 'deepObject',
        required: false,
        schema: {
            type: 'object',
            examples: [
                {
                    number: constants_1.DEFAULT_QUERY_PAGE,
                    size: constants_1.DEFAULT_PAGE_SIZE,
                },
            ],
            properties: {
                number: {
                    type: 'integer',
                    minimum: 1,
                },
                size: {
                    type: 'integer',
                    minimum: 1,
                    maximum: 500,
                },
            },
            additionalProperties: false,
        },
        description: `"${entity.name}" resource has been limit and offset with this params.`,
    })(controller, methodName, descriptor);
    (0, swagger_1.ApiResponse)({
        status: 400,
        description: 'Wrong query parameters',
        schema: utils_1.errorSchema,
    })(controller, methodName, descriptor);
    (0, swagger_1.ApiResponse)({
        status: 200,
        description: 'Resource list received successfully',
        schema: (0, utils_1.jsonSchemaResponse)(entity, mapEntity, true),
    })(controller, methodName, descriptor);
}
//# sourceMappingURL=get-all.js.map